<?php

if (!defined('ABSPATH')) exit;

class WooCommerce_Shipmate_Consignment
{

    protected $sender_reference = null;
    protected $delivery_instructions = "";
    protected $delivery_service = null;
    protected $delivery_service_key = "";
    protected $dispatch_date = null;
    protected $to_address = null;
    protected $parcels = array();

    /**
     * Constructor function.
     * @access  public
     * @return  void
     * @since   1.0.0
     */
    public function __construct($wc_order, $parcel_dimensions, $delivery_service_key = "", $dispatch_date = null)
    {
        if (WooCommerce_Shipmate_Integration::version_check('3.0')) {
            $order_data = $wc_order->get_data();
        } else {
            $order_data = [
                'shipping' => $wc_order->get_address('shipping'),
                'billing' => $wc_order->get_address('billing'),
                'customer_note' => $wc_order->customer_note,
                'id' => $wc_order->id,
                'shipping_lines' => $wc_order->get_shipping_methods()
            ];
        }


        $this->sender_reference = $order_data['id'];
        $this->to_address = new WooCommerce_Shipmate_Address($order_data['shipping'], $order_data['billing']);
        $this->dispatch_date = $dispatch_date;
        $this->delivery_instructions = $order_data['customer_note'];

        $delivery_service = null;
        if (!empty($delivery_service_key)) {
            $this->delivery_service_key = $delivery_service_key;
        } else if (!empty($order_data['shipping_lines'])) {
            foreach ($order_data['shipping_lines'] as $shipping_line) {
                if (WooCommerce_Shipmate_Integration::version_check('3.0')) {
                    $method_id = $shipping_line->get_method_id();
                    $method_key = explode(':', $method_id)[0];
                    $method_service = get_option("_shipmate_delivery_service_" . $method_key, null);

                    if ($method_service) {
                        $delivery_service = $method_service;
                    }
                } else {

                    $method_id = $shipping_line['method_id'];
                    $method_key = explode(':', $method_id)[0];
                    $method_service = get_option("_shipmate_delivery_service_" . $method_key, null);

                    if ($method_service) {
                        $delivery_service = $method_service;
                    }

                }

            }

        }


        // check we have been supplied parcel dimensions somehow
        if (!empty($parcel_dimensions) && !empty($parcel_dimensions['parcel_weights']) && !empty($parcel_dimensions['parcel_lengths']) && !empty($parcel_dimensions['parcel_widths']) && !empty($parcel_dimensions['parcel_depths'])) {

            $number_of_parcels = 0;
            foreach ($parcel_dimensions['parcel_weights'] as $parcel_weight) {
                if (!empty($parcel_weight)) {
                    $number_of_parcels += 1;
                }
            }

            $parcel_num = 1;

            for ($i = 0; $i < $number_of_parcels; $i++) {
                $productsMeta = $this->get_products_meta($wc_order);

                $this->parcels[] = new WooCommerce_Shipmate_Parcel($parcel_num, $order_data['id'], $parcel_dimensions['parcel_weights'][$i], $parcel_dimensions['parcel_lengths'][$i], $parcel_dimensions['parcel_widths'][$i], $parcel_dimensions['parcel_depths'][$i], array('DeliveryService' => $delivery_service), $productsMeta);

                $parcel_num += 1;
            }
        }
    } // End __construct ()

    public function toArray()
    {
        $consignment = array(
            'consignment_reference' => $this->sender_reference(),
            'dispatch_date' => $this->dispatch_date(),
            'to_address' => array(),
            'parcels' => array(),
            'service_key' => $this->delivery_service_key(),
            'delivery_instructions' => $this->delivery_instructions(),
        );

        foreach ($this->parcels() as $parcel) {
            $consignment['parcels'][] = $parcel->toArray();
        }

        $consignment['to_address'] = $this->to_address()->toArray();

        return $consignment;
    }

    /**
     * Gets the value of sender_reference.
     *
     * @return mixed
     */
    public function sender_reference()
    {
        return $this->sender_reference;
    }

    /**
     * Gets the value of dispatch_date.
     *
     * @return mixed
     */
    public function dispatch_date()
    {
        return $this->dispatch_date;
    }

    /**
     * Gets the value of delivery_service.
     *
     * @return mixed
     */
    public function delivery_service()
    {
        return $this->delivery_service;
    }

    /**
     * Gets the value of delivery_service_key.
     *
     * @return mixed
     */
    public function delivery_service_key()
    {
        return $this->delivery_service_key;
    }

    public function set_delivery_service_key($delivery_service_key)
    {
        $this->delivery_service_key = $delivery_service_key;
    }

    /**
     * Gets the value of to_address.
     *
     * @return mixed
     */
    public function to_address()
    {
        return $this->to_address;
    }

    /**
     * Gets the value of parcels.
     *
     * @return mixed
     */
    public function parcels()
    {
        return $this->parcels;
    }

    public function delivery_instructions()
    {
        return $this->delivery_instructions;
    }


    private function get_products_meta($wc_order)
    {
        $productDataHsCode = [];
        $productDataShortDescription = [];
        $productDataFullDescription = [];
        $productDataCountryOfOrigin = [];
        $productDataWeight = [];
        $productDataValue = [];
        $productItemValue = [];
        $productItemQuantity = [];
        $skuList = [];

        $items = array();


        foreach ($wc_order->get_items() as $item_id => $item) {

            if (WooCommerce_Shipmate_Integration::version_check('3.0')) {
                $productId = $item->get_product_id();
                $qty = $item->get_quantity();


                // If the product is a variation, get variation product regualr price else get the shipping item price
                if ($item->get_product()->is_type('variation')) {
                    $total = $item->get_product()->get_price();
                    // Also get the product _shipmate_item_value if set
                    $totalFromShipmate = get_post_meta($productId, '_shipmate_item_value', true);
                    // If the _shipmate_item_value is set and is > 0, use it instead of the product regular price
                    if (!empty($totalFromShipmate) && $totalFromShipmate > 0) {
                        $total = $totalFromShipmate;
                    }

                } else {
                    $total = get_post_meta($productId, '_shipmate_item_value', true);
                }
            } else {

                $productId = $item['product_id'];

                $qty = $item['qty'];
                $total = $item['line_total'];
            }

            // Add item data only if not empty
            $itemData = [];
            if(!empty(get_post_meta($productId, '_shipmate_hs_code', true))) {
                $itemData['harmonised_code'] = get_post_meta($productId, '_shipmate_hs_code', true);
            }

            if(!empty(get_post_meta($productId, '_shipmate_short_description', true))) {
                $itemData['short_description'] = get_post_meta($productId, '_shipmate_short_description', true);
            } else {
                $itemData['short_description'] = $item->get_name();
            }

            if(!empty(get_post_meta($productId, '_shipmate_full_description', true))) {
                $itemData['full_description'] = get_post_meta($productId, '_shipmate_full_description', true);
            } else {
                $itemData['full_description'] = $item->get_name();
            }

            if(!empty(get_post_meta($productId, '_shipmate_country_of_origin', true))) {
                $itemData['country_of_origin'] = get_post_meta($productId, '_shipmate_country_of_origin', true);
            }

            $itemData['item_value'] = $total;
            

            if(!empty($itemData)) {
                $itemData['item_quantity'] = $qty;
            }

            $itemData['sku'] = $item->get_product()->get_sku();
            $items[] = $itemData;

            // $productDataHsCode[] = get_post_meta($productId, '_shipmate_hs_code', true);
            // $productDataShortDescription[] = get_post_meta($productId, '_shipmate_short_description', true);
            // $productDataFullDescription[] = get_post_meta($productId, '_shipmate_full_description', true);
            // $productDataCountryOfOrigin[] = get_post_meta($productId, '_shipmate_country_of_origin', true);
            // $productItemValue[] = get_post_meta($productId, '_shipmate_item_value', true);
            // $productItemQuantity[] = $qty;
            // $productDataValue[] = $total;
            // $productDataWeight[] = $this->weight_converter($productWeight) * $qty;
            // $skuList[] = get_post_meta($productId, '_sku', true);
        }

       return $items;

    }


    private function weight_converter($weight)
    {
        $weight_unit = get_option('woocommerce_weight_unit');

        switch ($weight_unit) {
            case 'g':
                return $weight;
            case 'kg':
                return $weight * 1000;
            case 'lbs':
                return $weight * 454;
            case 'oz':
                return $weight * 28.35;
        }
    }
}
